<?php

namespace Drupal\video_toolbox\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\video_toolbox\VideoHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration Form for the video_toolbox module.
 */
class VideoToolBoxConfigForm extends ConfigFormBase {

  /**
   * Video Handler Service.
   *
   * @var \Drupal\video_toolbox\VideoHandlerInterface
   */
  protected $videoHandler;

  /**
   * Constructor to initialize Services.
   */
  public function __construct(VideoHandlerInterface $videoHandler) {
    $this->videoHandler = $videoHandler;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('video.get_service'),
      $container->get('messenger'),
    );
  }

  /**
   * Config settings for video_toolbox module.
   *
   * @var string
   */
  const VIDEO_TOOLBOX_SETTINGS = 'video_toolbox.settings';

  /**
   * Default configurations for the Video Player, colors are hexadecimal.
   *
   * @var string
   */
  const DEFAULT_PLAYER_SETTINGS = [
    'default_control_color' => 0,
    'control_color' => '#ffffff',
    'bar_color' => '#010141',
    'progress_color' => '#cccccc',
    'button_color' => '#ffffff',
  ];

  /**
   * Extensions.
   *
   * @var array
   */
  const EXT = [
    "avi",
    "flv",
    "mkv",
    "mov",
    "mp4",
    "webm",
  ];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'video_toolbox_configuration_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      self::VIDEO_TOOLBOX_SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(self::VIDEO_TOOLBOX_SETTINGS);
    $controlColor = $config->get('video_player.control_color');

    // In case the configuration is set to "none", change this variable
    // to the default color white.
    if ($controlColor === 'none') {
      $controlColor = self::DEFAULT_PLAYER_SETTINGS['control_color'];
    }

    $form['forceUpdate'] = [
      '#type' => 'checkbox',
      '#description' => $this->t('Force import'),
    ];

    $form['import'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import all videos'),
      '#attributes' => [
        'class' => ['button button--primary'],
      ],
      '#submit' => ['::import'],
    ];

    $form['allowed_extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed extensions'),
      '#description' => $this->t('This is for the video extensions that you can allowed, separate them witha comma'),
      '#default_value' => $config->get('allowed_extensions') ?? 'mp4',
      '#required' => 'true',
    ];

    $form['public'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Public video folder'),
      '#description' => $this->t('Make sure that the structure is public://{custom folder}/, ex "public://epic/"'),
      '#default_value' => $config->get('folder_pub') ?? 'public://video_saving/',
      '#required' => 'true',
    ];

    $form['private'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Private video folder'),
      '#description' => $this->t('Make sure that the structure is private://{custom folder}/, ex "private://epic/", for private paths please setup a path to <br> in /admin/config/media/file-system and create a folder inside the private path with the name of your {custom folder}'),
      '#default_value' => $config->get('folder_priv') ?? 'private://video_hidden_saving/',
      '#required' => 'true',
    ];

    $form['enable_custom_player'] = [
      '#type' => 'radios',
      '#title' => $this->t('Custom Player'),
      '#default_value' => $config->get('video_player.enable_custom_player'),
      '#options' => [
        1 => $this->t('Enable'),
        0 => $this->t('Disable'),
      ],
    ];

    $form['video_player_config'] = [
      '#type' => 'fieldgroup',
      '#title' => $this->t('Customize Player'),
      '#states' => [
        'visible' => [
          ':input[name="enable_custom_player"]' => ['value' => 1],
        ],
      ],
    ];

    $form['video_player_config']['set_default_btn'] = [
      '#prefix' => '<hr>',
      '#type' => 'submit',
      '#title' => $this->t('Set default configuration for Video player'),
      '#value' => $this->t('Set Default Video Player Settings'),
      '#attributes' => [
        'class' => ['button button--primary'],
      ],
      '#submit' => ['::setDefaultVideoPlayer'],
    ];

    $form['video_player_config']['default_control_color'] = [
      '#type' => 'radios',
      '#title' => $this->t('Choose default for controls background'),
      '#default_value' => $config->get('video_player.default_control_color'),
      '#options' => [
        0 => $this->t('Transparency'),
        1 => $this->t('Choose Color'),
      ],
    ];

    $form['video_player_config']['control_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Video Control Color'),
      '#default_value' => $controlColor,
      '#states' => [
        'visible' => [
          ':input[name="default_control_color"]' => ['value' => 1],
        ],
      ],
    ];

    $form['video_player_config']['bar_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Video Bar Color'),
      '#default_value' => $config->get('video_player.bar_color'),
    ];

    $form['video_player_config']['progress_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Video Progress Bar Color'),
      '#default_value' => $config->get('video_player.progress_color'),
    ];

    $form['video_player_config']['button_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Video Buttons Color'),
      '#default_value' => $config->get('video_player.button_color'),
    ];

    $form['video_player_config']['preview_div'] = [
      '#type' => 'video_player',
      '#title' => $this->t('Preview of Video Player Style'),
      '#width' => '500px',
      '#height' => '375px',
      '#preview' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    // Check for extensions.
    $ext = explode(',', preg_replace('/[ ]+/', '', $form_state->getValue('allowed_extensions')));

    if (count(array_intersect($ext, self::EXT)) != count($ext)) {
      $form_state->setErrorByName('allowed_extensions', $this->t('Invalid extension!'));
    }
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save configuration on submit.
    $controlDefault = $form_state->getValue('default_control_color');
    $config = $this->config(self::VIDEO_TOOLBOX_SETTINGS);
    $config
      ->set('allowed_extensions', $form_state->getValue('allowed_extensions'))
      ->set('folder_pub', $form_state->getValue('public'))
      ->set('folder_priv', $form_state->getValue('private'))
      ->set('video_player.enable_custom_player', $form_state->getValue('enable_custom_player'))
      ->set('video_player.bar_color', $form_state->getValue('bar_color'))
      ->set('video_player.progress_color', $form_state->getValue('progress_color'))
      ->set('video_player.button_color', $form_state->getValue('button_color'));
    // Depending on the Transparency or Color choice.
    if ($controlDefault == 0) {
      $config->set('video_player.control_color', 'none');
      $config->set('video_player.default_control_color', 0);
    }
    else {
      $config->set('video_player.control_color', $form_state->getValue('control_color'));
      $config->set('video_player.default_control_color', 1);
    }
    $config->save();
    drupal_flush_all_caches();
    parent::submitForm($form, $form_state);
  }

  /**
   * Import.
   */
  public function import(array &$form, FormStateInterface $form_state) {
    $force = FALSE;
    if ($form_state->getValue('forceUpdate') == 1) {
      $force = TRUE;
    }
    $import = $this->videoHandler->import($force);

    if ($import) {
      $this->messenger()->addMessage($this->t('Videos imported successfully'));
    }
    else {
      $this->messenger()->addWarning($this->t('Nothing to import'));
    }

  }

  /**
   * Set the default values for the Video Player settings.
   */
  public function setDefaultVideoPlayer(array &$form, FormStateInterface $form_state) {
    $form_state->setValue('default_control_color', self::DEFAULT_PLAYER_SETTINGS['default_control_color']);
    $form_state->setValue('control_color', self::DEFAULT_PLAYER_SETTINGS['control_color']);
    $form_state->setValue('bar_color', self::DEFAULT_PLAYER_SETTINGS['bar_color']);
    $form_state->setValue('progress_color', self::DEFAULT_PLAYER_SETTINGS['progress_color']);
    $form_state->setValue('button_color', self::DEFAULT_PLAYER_SETTINGS['button_color']);
    $this->submitForm($form, $form_state);
  }

}
